==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/mybounce.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: mybounce.c,v 1.4 2003/02/11 13:43:56 lwa Exp $";

#include "config.h"

#include <syslog.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "lmtp.h"

static char const default_msg[] = 
  "Content-Description: Notification\n"
  "Content-Type: text/plain\n"
  "\n"
  "Your message below cannot be delivered to one or more recipient for\n"
  "the following reason\n"
  "%s\n"
  "\n";

int mybounce_init(struct fstate *state) {
  int i;
  char *tofree;
  if (!state->mybounce)
    return 0; /* nothing to do */
  if (state->mybounce_msg.argc == 0) {
    state->mybounce_msg.argv[0] = (char *)default_msg;
    state->mybounce_msg.argc = 1;
    return 0;
  }
  tofree = state->mybounce_msg.argv[0];
  for (i=0; i<state->mybounce_msg.argc; i++) {
    char filename[MAXPATHLEN];
    FILE *f;
    int len;
    char *msg = NULL;
    if (state->mybounce_path && state->mybounce_path[0] && 
	state->mybounce_msg.argv[i][0] != '/') 
      strfmt(filename, sizeof(filename)-1, state->mybounce_path,
	     "$s", state->mybounce_msg.argv[i]);
    else 
      strncpy(filename, state->mybounce_msg.argv[i], sizeof(filename)-1);
    filename[sizeof(filename)-1] = 0;
    if ((f = fopen(filename, "r")) == NULL) {
      syslog(LOG_ERR, "unable to read %s: %s", filename, strerror(errno));
      return -1;
    }
    if (fseek(f, 0, SEEK_END) == -1) {
      syslog(LOG_ERR, "error seeking %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }
      
    if ((len = ftell(f)) == -1) {
      syslog(LOG_ERR, "error ftelling %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }

    if ((msg = malloc(len + 1)) == NULL) {
      syslog(LOG_ERR, "memory allocation error: %s", strerror(errno));
      fclose(f);
      return -1;
    }

    if (fseek(f, 0, SEEK_SET) == -1) {
      syslog(LOG_ERR, "error seeking %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }

    if (fread(msg, 1, len, f) != len) {
      syslog(LOG_ERR, "invalid size in %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }

    fclose(f);
    msg[len] = 0;
    state->mybounce_msg.argv[i] = msg;
  }
  free(tofree);
  return 0;
}

static void mybounce_msg1(struct fstate *state, FILE *f, char *msg) {
  char *start = msg;
  while(*msg) {
    if (*msg == '%') {
      int i;
      if (msg > start)
	fwrite(start, 1, msg - start, f);
      msg ++;
      switch(*msg) {
      case '%':
	putc('%', f);
	break;
      case 'h':
	fputs(state->myhostname, f);
	break;
      case 'u':
	for (i=0; i<state->nrcpts; i++) {
	  struct rcpt *rcpt = state->rcpts+i;
	  if (rcpt->bounce)
	    fprintf(f, " %s\n", rcpt->user);
	}
	break;
      case 's':
	for (i=0; i<state->nrcpts; i++) {
	  struct rcpt *rcpt = state->rcpts+i;
	  if (rcpt->bounce)
	    fprintf(f, " %s", rcpt->bounce); /* ended by \n */
	}
	break;
      }
      start = msg + 1;
    }
    msg ++;
  }
  if (msg > start)
    fwrite(start, 1, msg - start - 1, f);
  putc('\n', f);
}

static void mybounce_msg(struct fstate *state, FILE *f, char *boundary) {
  int i;
  if (state->mybounce_msg.argc == 1) {
    mybounce_msg1(state, f, state->mybounce_msg.argv[0]);
    return;
  }
  fprintf(f, "Content-Description: Notification\n");
  fprintf(f, "Content-Type: multipart/alternative;\n"
	  "\tboundary=\"=%s\"\n\n", boundary);
  for (i = 0; i< state->mybounce_msg.argc; i++) {
    fprintf(f, "--=%s\n", boundary);
    mybounce_msg1(state, f, state->mybounce_msg.argv[i]);
  }
  fprintf(f, "--=%s--\n", boundary);
}

int mybounce(struct fstate *state) {
  int fd = runsendmail(NULL, state->sendmail_path, "", state->sender);
  FILE *f;
  char boundary[LINELEN];
  char bufdate[100];
  int i;
  size_t bodysize;

  if (fd == -1)
    return -1;
  f = fdopen(fd, "w");
  if (f == NULL) {
    close(fd);
    return -1;
  }
  strftime(bufdate, sizeof(bufdate)-1, STRFTIME_FMT,
           localtime(&(state->date)));
  snprintf(boundary, sizeof(boundary)-1, "%lX.%lu.%lu/%s",
	   (unsigned long)state->date, (unsigned long)state->pid,
	   (unsigned long)state->counter, state->myhostname); 
  fprintf(f, "From: MAILER-DAEMON (Mail Delivery System)\n");
  fprintf(f, "Subject: Undelivered Mail Returned to Sender\n");
  fprintf(f, "To: state->sender\n");
  fprintf(f, "Date: %s\n", bufdate);
  fprintf(f, "Message-ID: <%lX-%lu." MID_FMT  ".%s@%s>\n",
          (unsigned long)state->date, (unsigned long)state->pid,
	  state->counter,  "bounce", state->myhostname);
  fprintf(f, "MIME-Version: 1.0\n"); 
  fprintf(f, "Content-Type: multipart/report; report-type=delivery-status;\n"
	  "\tboundary=\"%s\"\n", boundary);
  fprintf(f, "\n"); /* end of headers */

  fprintf(f, "This is a MIME-encapsulated message.\n\n");

  fprintf(f, "--%s\n", boundary);
  
  mybounce_msg(state, f, boundary);

  fprintf(f, "--%s\n", boundary);
  fprintf(f, "Content-Description: Delivery Status Notification\n");
  fprintf(f, "Content-Type: message/delivery-status\n");
  fprintf(f, "\n"); /* end of sub-headers */

  fprintf(f, "Reporting-MTA: dns; %s\n", state->myhostname);
  if (state->lhlo)
    fprintf(f, "Received-From-MTA: lmtp; %s\n", state->lhlo);
  fprintf(f, "Arrival-Date: %s\n", bufdate);
  fprintf(f, "\n"); /* end of sub-sub-headers */

  for (i=0; i<state->nrcpts; i++) {
    struct rcpt *rcpt1 = state->rcpts + i;
    if (rcpt1->bounce) {
      char *xbounce = rcpt1->bounce + 4;
      char *ebounce = xbounce;
      while(*ebounce && *ebounce != ' ') ebounce ++;
      fprintf(f, "Final-Recipient: rfc822; %s\n", rcpt1->user);
      fprintf(f, "Action: failed\n");
      fprintf(f, "Status: %.*s\n", (int)(ebounce-xbounce), xbounce);
      fprintf(f, "Diagnostic-Code: smtp; %s", rcpt1->bounce);
      fprintf(f, "\n"); /* end of sub-sub-headers */
    }
  }

  fprintf(f, "--%s\n", boundary);
  fprintf(f, "Content-Description: Original message headers\n");
  fprintf(f, "Content-Type: text/rfc822-headers\n");
  fprintf(f, "\n"); /* end of sub-headers */
  fflush(f);

  /* send headers only */
  bodysize = state->bodysize;
  state->bodysize = 0;
  mailwrite(state, state->hdr_my_received->next, HEADER_OTHER, fd);
  state->bodysize = bodysize;

  fprintf(f, "\n--%s--\n", boundary);
  fclose(f);
  waitsendmail(NULL, -1);
  return 0;
}

==> lmtpd-0.10.7.tar.gz: lmtpd-0.10.7/mybounce.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: mybounce.c,v 1.5 2004/10/26 09:59:20 lwa Exp $";



#include <syslog.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "lmtp.h"

static char const default_msg[] = 
  "Content-Description: Notification\n"
  "Content-Type: text/plain\n"
  "\n"
  "Your message below cannot be delivered to one or more recipient for\n"
  "the following reason\n"
  "%s\n"
  "\n";

int mybounce_init(struct fstate *state) {
  int i;
  char *tofree;
  if (!state->mybounce)
    return 0; /* nothing to do */
  if (state->mybounce_msg.argc == 0) {
    state->mybounce_msg.argv[0] = (char *)default_msg;
    state->mybounce_msg.argc = 1;
    return 0;
  }
  tofree = state->mybounce_msg.argv[0];
  for (i=0; i<state->mybounce_msg.argc; i++) {
    char filename[MAXPATHLEN];
    FILE *f;
    int len;
    char *msg = NULL;
    if (state->mybounce_path && state->mybounce_path[0] && 
	state->mybounce_msg.argv[i][0] != '/') 
      strfmt(filename, sizeof(filename)-1, state->mybounce_path,
	     "$s", state->mybounce_msg.argv[i]);
    else 
      strncpy(filename, state->mybounce_msg.argv[i], sizeof(filename)-1);
    filename[sizeof(filename)-1] = 0;
    if ((f = fopen(filename, "r")) == NULL) {
      syslog(LOG_ERR, "unable to read %s: %s", filename, strerror(errno));
      return -1;
    }
    if (fseek(f, 0, SEEK_END) == -1) {
      syslog(LOG_ERR, "error seeking %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }
      
    if ((len = ftell(f)) == -1) {
      syslog(LOG_ERR, "error ftelling %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }

    if ((msg = malloc(len + 1)) == NULL) {
      syslog(LOG_ERR, "memory allocation error: %s", strerror(errno));
      fclose(f);
      return -1;
    }

    if (fseek(f, 0, SEEK_SET) == -1) {
      syslog(LOG_ERR, "error seeking %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }

    if (fread(msg, 1, len, f) != len) {
      syslog(LOG_ERR, "invalid size in %s: %s", filename, strerror(errno));
      fclose(f);
      return -1;
    }

    fclose(f);
    msg[len] = 0;
    state->mybounce_msg.argv[i] = msg;
  }
  free(tofree);
  return 0;
}

static void mybounce_msg1(struct fstate *state, FILE *f, char *msg) {
  char *start = msg;
  while(*msg) {
    if (*msg == '%') {
      int i;
      if (msg > start)
	fwrite(start, 1, msg - start, f);
      msg ++;
      switch(*msg) {
      case '%':
	putc('%', f);
	break;
      case 'h':
	fputs(state->myhostname, f);
	break;
      case 'u':
	for (i=0; i<state->nrcpts; i++) {
	  struct rcpt *rcpt = state->rcpts+i;
	  if (rcpt->bounce)
	    fprintf(f, " %s\n", rcpt->user);
	}
	break;
      case 's':
	for (i=0; i<state->nrcpts; i++) {
	  struct rcpt *rcpt = state->rcpts+i;
	  if (rcpt->bounce)
	    fprintf(f, " %s", rcpt->bounce); /* ended by \n */
	}
	break;
      }
      start = msg + 1;
    }
    msg ++;
  }
  if (msg > start)
    fwrite(start, 1, msg - start - 1, f);
  putc('\n', f);
}

static void mybounce_msg(struct fstate *state, FILE *f, char *boundary) {
  int i;
  if (state->mybounce_msg.argc == 1) {
    mybounce_msg1(state, f, state->mybounce_msg.argv[0]);
    return;
  }
  fprintf(f, "Content-Description: Notification\n");
  fprintf(f, "Content-Type: multipart/alternative;\n"
	  "\tboundary=\"=%s\"\n\n", boundary);
  for (i = 0; i< state->mybounce_msg.argc; i++) {
    fprintf(f, "--=%s\n", boundary);
    mybounce_msg1(state, f, state->mybounce_msg.argv[i]);
  }
  fprintf(f, "--=%s--\n", boundary);
}

int mybounce(struct fstate *state) {
  int fd = runsendmail(NULL, state->sendmail_path, "", state->sender);
  FILE *f;
  char boundary[LINELEN];
  char bufdate[100];
  int i;
  size_t bodysize;

  if (fd == -1)
    return -1;
  f = fdopen(fd, "w");
  if (f == NULL) {
    close(fd);
    return -1;
  }
  strftime(bufdate, sizeof(bufdate)-1, STRFTIME_FMT,
           localtime(&(state->date)));
  snprintf(boundary, sizeof(boundary)-1, "%lX.%lu.%lu/%s",
	   (unsigned long)state->date, (unsigned long)state->pid,
	   (unsigned long)state->counter, state->myhostname); 
  fprintf(f, "From: MAILER-DAEMON (Mail Delivery System)\n");
  fprintf(f, "Subject: Undelivered Mail Returned to Sender\n");
  fprintf(f, "To: state->sender\n");
  fprintf(f, "Date: %s\n", bufdate);
  fprintf(f, "Message-ID: <%lX-%lu." MID_FMT  ".%s@%s>\n",
          (unsigned long)state->date, (unsigned long)state->pid,
	  state->counter,  "bounce", state->myhostname);
  fprintf(f, "MIME-Version: 1.0\n"); 
  fprintf(f, "Content-Type: multipart/report; report-type=delivery-status;\n"
	  "\tboundary=\"%s\"\n", boundary);
  fprintf(f, "\n"); /* end of headers */

  fprintf(f, "This is a MIME-encapsulated message.\n\n");

  fprintf(f, "--%s\n", boundary);
  
  mybounce_msg(state, f, boundary);

  fprintf(f, "--%s\n", boundary);
  fprintf(f, "Content-Description: Delivery Status Notification\n");
  fprintf(f, "Content-Type: message/delivery-status\n");
  fprintf(f, "\n"); /* end of sub-headers */

  fprintf(f, "Reporting-MTA: dns; %s\n", state->myhostname);
  if (state->lhlo)
    fprintf(f, "Received-From-MTA: lmtp; %s\n", state->lhlo);
  fprintf(f, "Arrival-Date: %s\n", bufdate);
  fprintf(f, "\n"); /* end of sub-sub-headers */

  for (i=0; i<state->nrcpts; i++) {
    struct rcpt *rcpt1 = state->rcpts + i;
    if (rcpt1->bounce) {
      char *xbounce = rcpt1->bounce + 4;
      char *ebounce = xbounce;
      while(*ebounce && *ebounce != ' ') ebounce ++;
      fprintf(f, "Final-Recipient: rfc822; %s\n", rcpt1->user);
      fprintf(f, "Action: failed\n");
      fprintf(f, "Status: %.*s\n", (int)(ebounce-xbounce), xbounce);
      fprintf(f, "Diagnostic-Code: smtp; %s", rcpt1->bounce);
      fprintf(f, "\n"); /* end of sub-sub-headers */
    }
  }

  fprintf(f, "--%s\n", boundary);
  fprintf(f, "Content-Description: Original message headers\n");
  fprintf(f, "Content-Type: text/rfc822-headers\n");
  fprintf(f, "\n"); /* end of sub-headers */
  fflush(f);

  /* send headers only */
  bodysize = state->bodysize;
  state->bodysize = 0;
  mailwrite(state, state->hdr_my_received->next, HEADER_OTHER, fd);
  state->bodysize = bodysize;

  fprintf(f, "\n--%s--\n", boundary);
  fclose(f);
  waitsendmail(NULL, -1);
  return 0;
}
